/*
 * Copyright (c) 2002-2008, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.helpdesk.service.search;

import fr.paris.lutece.plugins.helpdesk.business.Faq;
import fr.paris.lutece.plugins.helpdesk.business.FaqHome;
import fr.paris.lutece.plugins.helpdesk.business.QuestionAnswer;
import fr.paris.lutece.plugins.helpdesk.business.Subject;
import fr.paris.lutece.plugins.helpdesk.business.SubjectHome;
import fr.paris.lutece.plugins.helpdesk.service.HelpdeskPlugin;
import fr.paris.lutece.plugins.helpdesk.service.helpdesksearch.HelpdeskSearchItem;
import fr.paris.lutece.plugins.helpdesk.web.HelpdeskApp;
import fr.paris.lutece.portal.service.content.XPageAppService;
import fr.paris.lutece.portal.service.message.SiteMessageException;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.plugin.PluginService;
import fr.paris.lutece.portal.service.search.SearchIndexer;
import fr.paris.lutece.portal.service.search.SearchItem;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.util.url.UrlItem;

import org.apache.lucene.demo.html.HTMLParser;
import org.apache.lucene.document.DateTools;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;


/**
 * Helpdesk Indexer
 *
 */
public class HelpdeskIndexer implements SearchIndexer
{
    private static final String ENABLE_VALUE_TRUE = "1";
    private static final String PROPERTY_PAGE_PATH_LABEL = "helpdesk.pagePathLabel";
    private static final String PROPERTY_INDEXER_DESCRIPTION = "helpdesk.indexer.description";
    private static final String PROPERTY_INDEXER_NAME = "helpdesk.indexer.name";
    private static final String PROPERTY_INDEXER_VERSION = "helpdesk.indexer.version";
    private static final String PROPERTY_INDEXER_ENABLE = "helpdesk.indexer.enable";

    /**
     * Returns the indexer service description
     * @return The indexer service description
     */
    public String getDescription(  )
    {
        return AppPropertiesService.getProperty( PROPERTY_INDEXER_DESCRIPTION );
    }

    /**
     * Get the document list
     *
     * @return The list of lucene documents
     * @throws IOException
     * @throws InterruptedException
     * @throws SiteMessageException
     */
    public List<Document> getDocuments(  ) throws IOException, InterruptedException, SiteMessageException
    {
        List<org.apache.lucene.document.Document> listDocs = new ArrayList<org.apache.lucene.document.Document>(  );
        String strPortalUrl = AppPathService.getPortalUrl(  );
        Plugin plugin = PluginService.getPlugin( HelpdeskPlugin.PLUGIN_NAME );

        for ( Faq faq : FaqHome.findAll( plugin ) )
        {
            for ( Subject subject : (Collection<Subject>) SubjectHome.getInstance(  ).findByIdFaq( faq.getId(  ), plugin ) )
            {
                UrlItem urlSubject = new UrlItem( strPortalUrl );
                urlSubject.addParameter( XPageAppService.PARAM_XPAGE_APP,
                    AppPropertiesService.getProperty( PROPERTY_PAGE_PATH_LABEL ) ); //FIXME
                urlSubject.addParameter( HelpdeskApp.PARAMETER_FAQ_ID, faq.getId(  ) );
                urlSubject.setAnchor( HelpdeskApp.ANCHOR_SUBJECT + subject.getId(  ) );

                org.apache.lucene.document.Document docSubject = getDocument( subject, faq.getRoleKey(  ),
                        urlSubject.getUrl(  ), plugin );
                listDocs.add( docSubject );

                for ( QuestionAnswer questionAnswer : (List<QuestionAnswer>) subject.getQuestions(  ) )
                {
                    if ( questionAnswer.isEnabled(  ) )
                    {
                        UrlItem urlQuestionAnswer = new UrlItem( strPortalUrl );
                        urlQuestionAnswer.addParameter( XPageAppService.PARAM_XPAGE_APP,
                            AppPropertiesService.getProperty( PROPERTY_PAGE_PATH_LABEL ) ); //FIXME
                        urlQuestionAnswer.addParameter( HelpdeskApp.PARAMETER_FAQ_ID, faq.getId(  ) );
                        urlQuestionAnswer.setAnchor( HelpdeskApp.ANCHOR_QUESTION_ANSWER +
                            questionAnswer.getIdQuestionAnswer(  ) );

                        org.apache.lucene.document.Document docQuestionAnswer = getDocument( faq.getId(  ),
                                questionAnswer, urlQuestionAnswer.getUrl(  ), faq.getRoleKey(  ), plugin );
                        listDocs.add( docQuestionAnswer );
                    }
                }
            }
        }

        return listDocs;
    }

    /**
     * Builds a document which will be used by Lucene during the indexing of the question/answer list
     *
     * @param nIdFaq The {@link Faq} Id
     * @param questionAnswer the {@link QuestionAnswer} to index
     * @param strUrl the url of the subject
     * @param strRoleKey The role key
     * @param plugin The {@link Plugin}
     * @return A Lucene {@link Document} containing QuestionAnswer Data
     * @throws IOException The IO Exception
     * @throws InterruptedException The InterruptedException
     */
    public static org.apache.lucene.document.Document getDocument( int nIdFaq, QuestionAnswer questionAnswer,
        String strUrl, String strRoleKey, Plugin plugin )
        throws IOException, InterruptedException
    {
        // make a new, empty document
        org.apache.lucene.document.Document doc = new org.apache.lucene.document.Document(  );

        doc.add( new Field( HelpdeskSearchItem.FIELD_FAQ_ID, String.valueOf( nIdFaq ), Field.Store.YES,
                Field.Index.UN_TOKENIZED ) );

        doc.add( new Field( HelpdeskSearchItem.FIELD_ROLE, strRoleKey, Field.Store.YES, Field.Index.UN_TOKENIZED ) );

        // Add the url as a field named "url".  Use an UnIndexed field, so
        // that the url is just stored with the question/answer, but is not searchable.
        doc.add( new Field( HelpdeskSearchItem.FIELD_URL, strUrl, Field.Store.YES, Field.Index.UN_TOKENIZED ) );

        doc.add( new Field( HelpdeskSearchItem.FIELD_SUBJECT, String.valueOf( questionAnswer.getIdSubject(  ) ),
                Field.Store.YES, Field.Index.UN_TOKENIZED ) );

        // Add the uid as a field, so that index can be incrementally maintained.
        // This field is not stored with question/answer, it is indexed, but it is not
        // tokenized prior to indexing.
        String strIdQuestionAnswer = String.valueOf( questionAnswer.getIdQuestionAnswer(  ) );
        doc.add( new Field( HelpdeskSearchItem.FIELD_UID, strIdQuestionAnswer, Field.Store.YES, Field.Index.NO ) );

        // Add the last modified date of the file a field named "modified".
        // Use a field that is indexed (i.e. searchable), but don't tokenize
        // the field into words.
        String strDate = DateTools.dateToString( questionAnswer.getCreationDate(  ), DateTools.Resolution.DAY );
        doc.add( new Field( HelpdeskSearchItem.FIELD_DATE, strDate, Field.Store.YES, Field.Index.UN_TOKENIZED ) );

        String strContentToIndex = getContentToIndex( questionAnswer, plugin );
        StringReader readerPage = new StringReader( strContentToIndex );
        HTMLParser parser = new HTMLParser( readerPage );

        //the content of the question/answer is recovered in the parser because this one
        //had replaced the encoded caracters (as &eacute;) by the corresponding special caracter (as ?)
        Reader reader = parser.getReader(  );
        int c;
        StringBuffer sb = new StringBuffer(  );

        while ( ( c = reader.read(  ) ) != -1 )
        {
            sb.append( String.valueOf( (char) c ) );
        }

        reader.close(  );

        // Add the tag-stripped contents as a Reader-valued Text field so it will
        // get tokenized and indexed.
        doc.add( new Field( HelpdeskSearchItem.FIELD_CONTENTS, sb.toString(  ), Field.Store.NO, Field.Index.TOKENIZED ) );

        // Add the subject name as a separate Text field, so that it can be searched
        // separately.
        doc.add( new Field( HelpdeskSearchItem.FIELD_TITLE, questionAnswer.getQuestion(  ), Field.Store.YES,
                Field.Index.TOKENIZED ) );

        doc.add( new Field( HelpdeskSearchItem.FIELD_TYPE, HelpdeskPlugin.PLUGIN_NAME, Field.Store.YES,
                Field.Index.UN_TOKENIZED ) );

        // return the document
        return doc;
    }

    /**
     * Builds a document which will be used by Lucene during the indexing of the subject list
     *
     * @param subject the {@link Subject} to index
     * @param strUrl the url of the subject
     * @param strRoleKey The role key
     * @param plugin The {@link Plugin}
     * @return The Lucene {@link Document} containing Subject data
     * @throws IOException The IO Exception
     * @throws InterruptedException The InterruptedException
     */
    public static org.apache.lucene.document.Document getDocument( Subject subject, String strRoleKey, String strUrl,
        Plugin plugin ) throws IOException, InterruptedException
    {
        // make a new, empty document
        org.apache.lucene.document.Document doc = new org.apache.lucene.document.Document(  );

        // Add the url as a field named "url".  Use an UnIndexed field, so
        // that the url is just stored with the question/answer, but is not searchable.
        doc.add( new Field( SearchItem.FIELD_URL, strUrl, Field.Store.YES, Field.Index.UN_TOKENIZED ) );

        // Add the uid as a field, so that index can be incrementally maintained.
        // This field is not stored with question/answer, it is indexed, but it is not
        // tokenized prior to indexing.
        String strIdSubject = String.valueOf( subject.getId(  ) );
        doc.add( new Field( SearchItem.FIELD_UID, strIdSubject, Field.Store.NO, Field.Index.UN_TOKENIZED ) );

        doc.add( new Field( SearchItem.FIELD_CONTENTS, subject.getText(  ), Field.Store.NO, Field.Index.TOKENIZED ) );

        // Add the subject name as a separate Text field, so that it can be searched
        // separately.
        doc.add( new Field( SearchItem.FIELD_TITLE, subject.getText(  ), Field.Store.YES, Field.Index.UN_TOKENIZED ) );

        doc.add( new Field( SearchItem.FIELD_TYPE, HelpdeskPlugin.PLUGIN_NAME, Field.Store.YES, Field.Index.NO ) );

        doc.add( new Field( SearchItem.FIELD_ROLE, strRoleKey, Field.Store.YES, Field.Index.UN_TOKENIZED ) );

        // return the document
        return doc;
    }

    /**
     * Set the Content to index (Question and Answer)
     * @param questionAnswer The {@link QuestionAnswer} to index
     * @param plugin The {@link Plugin}
     * @return The content to index
     */
    private static String getContentToIndex( QuestionAnswer questionAnswer, Plugin plugin )
    {
        StringBuffer sbContentToIndex = new StringBuffer(  );
        //Do not index question here
        sbContentToIndex.append( questionAnswer.getQuestion(  ) );
        sbContentToIndex.append( questionAnswer.getAnswer(  ) );

        return sbContentToIndex.toString(  );
    }

    /**
     * Get the name of the indexer.
     * @return The name
     */
    public String getName(  )
    {
        return AppPropertiesService.getProperty( PROPERTY_INDEXER_NAME );
    }

    /**
     * Get the version of the indexer
     * @return The version number
     */
    public String getVersion(  )
    {
        return AppPropertiesService.getProperty( PROPERTY_INDEXER_VERSION );
    }

    /**
     * Get the state of indexer
     * @return Return true if the indexer is enabled, false else.
     */
    public boolean isEnable(  )
    {
        boolean bReturn = false;
        String strEnable = AppPropertiesService.getProperty( PROPERTY_INDEXER_ENABLE );

        if ( ( strEnable != null ) &&
                ( strEnable.equalsIgnoreCase( Boolean.TRUE.toString(  ) ) || strEnable.equals( ENABLE_VALUE_TRUE ) ) )
        {
            bReturn = true;
        }

        return bReturn;
    }
}
